/*
 * Component Name: Headset Simulator
 * Author: Comarch S.A.
 * Version: 1.0
 * Copyright (c) 2010 Comarch S.A.
 *  
 * This Software is submitted by Comarch S.A. to Symbian Foundation Limited on 
 * the basis of the Member Contribution Agreement entered between Comarch S.A. 
 * and Symbian Foundation Limited on 5th June 2009 (Agreement) and may be 
 * used only in accordance with the terms and conditions of the Agreement. 
 * Any other usage, duplication or redistribution of this Software is not 
 * allowed without written permission of Comarch S.A.
 * 
 */

#ifndef HSOBSERVER_H
#define HSOBSERVER_H

/**
 * @brief Observer to notify about Headset Simulator AG client and Remote 
 * Controller's states.
 */
class MHsObserver
{

public:
    /**
     * Informs about established connection with AG.
     */
    virtual void HandleConnectedToClient() = 0;

    /**
     * Informs about released connection with AG.
     */
    virtual void HandleDisconnectedFromClient() = 0;

    /**
     * Informs that connection with  AG cannot be established.
     */
    virtual void HandleUnableToConnectClient() = 0;

    /**
     * Informs that connection with  AG is being established.
     */
    virtual void HandleConnectingToClient() = 0;

    /**
     * Informs about established connection with Headset Simulator's 
     * Remote Controller.
     */
    virtual void HandleConnectedToRemoteController() = 0;

    /**
     * Informs about released connection with Headset Simulator's 
     * Remote Controller.
     */
    virtual void HandleDisconnectedFromRemoteController() = 0;

};

#endif //HSOBSERVER_H
